==> aspy.refactor_imports-2.1.0.tar.gz: aspy.refactor_imports-2.1.0/aspy/refactor_imports/sort.py <==
import collections
from typing import Any
from typing import Dict
from typing import List
from typing import Sequence
from typing import Tuple

from aspy.refactor_imports.classify import classify_import
from aspy.refactor_imports.classify import ImportType
from aspy.refactor_imports.import_obj import AbstractImportObj
from aspy.refactor_imports.import_obj import FromImport
from aspy.refactor_imports.import_obj import ImportImport


# For `import_before_from`
CLS_TO_INDEX = {ImportImport: '0', FromImport: '1'}


def sort(
        imports: Sequence[AbstractImportObj],
        separate: bool = True,
        import_before_from: bool = True,
        **kwargs: Any,
) -> Tuple[Tuple[AbstractImportObj, ...], ...]:
    """Sort import objects into groups.

    :param list imports: FromImport / ImportImport objects
    :param bool separate: Whether to classify and return separate segments
        of imports based on classification.
    :param bool import_before_from: Whether to sort `import ...` imports before
        `from ...` imports.

    For example:
        from os import path
        from aspy import refactor_imports
        import sys
        import pyramid

    separate = True, import_before_from = True

        import sys
        from os import path

        import pyramid

        from aspy import refactor_imports

    separate = True, import_before_from = False

        from os import path
        import sys

        import pyramid

        from aspy import refactor_imports

    separate = False, import_before_from = True

        import pyramid
        import sys
        from aspy import refactor_imports
        from os import path

    separate = False, import_before_from = False

        from aspy import refactor_imports
        from os import path
        import pyramid
        import sys
    """
    if separate:
        def classify_func(obj: AbstractImportObj) -> str:
            return classify_import(obj.import_statement.module, **kwargs)
        types: Tuple[str, ...] = ImportType.__all__
    else:
        # A little cheaty, this allows future imports to sort before others
        def classify_func(obj: AbstractImportObj) -> str:
            tp = classify_import(obj.import_statement.module, **kwargs)
            if tp == ImportType.FUTURE:
                return ImportType.FUTURE
            else:
                return '(not future)'
        types = (ImportType.FUTURE, '(not future)')

    if import_before_from:
        def sort_within(obj: AbstractImportObj) -> Tuple[str, ...]:
            return (CLS_TO_INDEX[type(obj)],) + obj.sort_key
    else:
        def sort_within(obj: AbstractImportObj) -> Tuple[str, ...]:
            return tuple(obj.sort_key)

    # Partition the imports
    imports_partitioned: Dict[str, List[AbstractImportObj]]
    imports_partitioned = collections.defaultdict(list)
    for import_obj in imports:
        imports_partitioned[classify_func(import_obj)].append(import_obj)

    # sort each of the segments
    for segment_key, val in imports_partitioned.items():
        imports_partitioned[segment_key] = sorted(val, key=sort_within)

    return tuple(
        tuple(imports_partitioned[key])
        for key in types if key in imports_partitioned
    )

==> aspy.refactor_imports-3.0.0.tar.gz: aspy.refactor_imports-3.0.0/aspy/refactor_imports/sort.py <==
from __future__ import annotations

import collections
from typing import Any
from typing import Sequence

from aspy.refactor_imports.classify import classify_import
from aspy.refactor_imports.classify import ImportType
from aspy.refactor_imports.import_obj import AbstractImportObj
from aspy.refactor_imports.import_obj import FromImport
from aspy.refactor_imports.import_obj import ImportImport


# For `import_before_from`
CLS_TO_INDEX = {ImportImport: '0', FromImport: '1'}


def sort(
        imports: Sequence[AbstractImportObj],
        **kwargs: Any,
) -> tuple[tuple[AbstractImportObj, ...], ...]:
    """Sort import objects into groups.

    :param list imports: FromImport / ImportImport objects

    For example:

        from os import path
        from aspy import refactor_imports
        import sys
        import pyramid

    becomes:

        import sys
        from os import path

        import pyramid

        from aspy import refactor_imports
    """
    # Partition the imports
    imports_partitioned: dict[str, list[AbstractImportObj]]
    imports_partitioned = collections.defaultdict(list)
    for import_obj in imports:
        tp = classify_import(import_obj.import_statement.module, **kwargs)
        if tp == ImportType.FUTURE and isinstance(import_obj, ImportImport):
            tp = ImportType.BUILTIN

        imports_partitioned[tp].append(import_obj)

    # sort each of the segments

    def sort_within(obj: AbstractImportObj) -> tuple[str, ...]:
        return (CLS_TO_INDEX[type(obj)],) + obj.sort_key

    for segment_key, val in imports_partitioned.items():
        imports_partitioned[segment_key] = sorted(val, key=sort_within)

    return tuple(
        tuple(imports_partitioned[key])
        for key in ImportType.__all__ if key in imports_partitioned
    )
